/*
 * qfits example: querying FITS headers, supporting extensions.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "qfits.h"

int main(int argc, char * argv[])
{
    char    keyword[FITS_LINESZ+1] ;
    char *  value ;
    int     xtnum ;
    int     n_ext ;

	if (argc!=3) {
		printf("use: %s <filename> <extension>\n", argv[0]);
		return 1 ;
	}
    /* Check this is indeed a FITS file */
    if (is_fits_file(argv[1])!=1) {
        printf("%s is not a FITS file\n", argv[1]);
        return -1 ;
    }
    /* Check the requested extension actually exists */
    xtnum = atoi(argv[2]);
    n_ext = qfits_query_n_ext(argv[1]);
    printf("file %s has %d extension(s)\n", argv[1], n_ext);
    if (xtnum<0 || xtnum>n_ext) {
        printf("extension %d does not exist\n", xtnum);
        return -1 ;
    }

    printf("\n\n");
    printf("file %s loaded Ok\n", argv[1]);
    printf("provide keywords to look for in extension header %d\n"
           "(extension 0 is main header)\n"
           "(q to quit)\n"
           "\n", xtnum);

    while (1) {
        printf("-----> keyword? ");
        scanf("%s", keyword);
        if (!strcmp(keyword, "q"))
            return 0 ;

        value = qfits_query_ext(argv[1], keyword, xtnum);
        if (value) {
            printf("found with value: %s\n", value);
        } else {
            printf("not found in header (%d)\n", xtnum);
        }
    }
	return 0 ;
}

