/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/**
   @file xmemory.pure.c
   @brief Basically a removal of all xmemory (Nicolas Devillard)
   functionality
   
   xmemory was meant to be an efficient module for memory handling,
   coming with the qfits library. For some purposes xmemory is a
   brilliant tool. However, not for all. This is a version of xmemory
   that takes away all functionality of xmemory, restoring all the old
   memory allocation functions. The only function qfits depends on,
   falloc is rewritten such that a normal free on a mapped file
   works. In course of compilation the old xmemory modules are removed
   and replaced with these.
   
   $Source: /Volumes/DATA_J_II/data/CVS/tirific/xmemory/xmemory.pure.c,v $
   $Date: 2009/05/26 07:56:41 $
   $Revision: 1.4 $
   $Author: jozsa $
   $Log: xmemory.pure.c,v $
   Revision 1.4  2009/05/26 07:56:41  jozsa
   Left work

   Revision 1.3  2007/07/03 17:28:08  gjozsa
   Left work

   Revision 1.2  2006/11/22 14:16:22  gjozsa
   Bugfix concerning RASH and horizontal/vertical lines

   Revision 1.1.1.1  2004/10/29 11:13:39  gjozsa
   Added to CVS control

   Revision 1.1.1.1  2004/10/28 14:47:36  gjozsa
   Added to CVS control

   
*/
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* EXTERNAL INCLUDES */
/* ------------------------------------------------------------ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* INTERNAL INCLUDES */
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* PRIVATE DEFINES */
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* GLOBAL VARIABLES */
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* PRIVATE TYPEDEFS */
/* ------------------------------------------------------------ */




/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* PRIVATE STRUCTS */
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* PRIVATE FUNCTIONS (DECLARATION) */
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* FUNCTION CODE */
/* ------------------------------------------------------------ */



/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/**
   @fn char *xmemory_falloc(char *name, size_t offs, size_t *size)
   @brief    Map a file's contents to memory as a char pointer.
   
   This function takes in input the name of a file. It tries to map the file 
  into memory and if it succeeds, returns the file's contents as a char pointer.
  It also modifies the input size variable to be the size of the mapped file in
  bytes. This function is normally never directly called but through the 
  falloc() macro.
  
  The offset indicates the starting point for the mapping, i.e. if you are not 
  interested in mapping the whole file but only from a given place.
  
  @param    name        Name of the file to map
   @param    offs        Offset to the first mapped byte in file.
   @param    size        Returned size of the mapped file in bytes.

   @return   A pointer to char, to be freed using free().
 */
/*----------------------------------------------------------------------------*/
char * xmemory_falloc(char *name, size_t offs, size_t *size)
{
  char *t;
  struct stat sta;
  FILE *thefile;

  if (stat(name, &sta)==-1) {
        return 0;
  }
  if ((t = (char *) malloc((sta.st_size-offs+1)*sizeof(char)))) {
    if ((thefile = fopen(name, "r"))) {
      fseek(thefile, offs, SEEK_SET);
      fread(t, sizeof(char), sta.st_size-offs+1, thefile);
      fclose(thefile);
      if (size!=NULL) {
        (*size) = sta.st_size ;
      }
    }
  }
  return t;
}

/* vim: set ts=4 et sw=4 tw=75 */
